--
-- AmazoneZAOC900
-- Specialization for AmazoneZAOC900
--
-- @author Felix "Outlaw" Sorge
-- @date  27/04/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
--

AmazoneZAOC900 = {};

function AmazoneZAOC900.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;


function AmazoneZAOC900:load(xmlFile)

	self.aiTerrainDetailChannel1 = g_currentMission.cultivatorChannel;
    self.aiTerrainDetailChannel2 = g_currentMission.sowingChannel;

	local count = getXMLInt(xmlFile, "vehicle.drums#count");
    self.drums = {}
	local part = self.drums;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.drums.part" .. "%d", i);
	    part[i].index = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname .. "#index"));
		local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partname .. "#rot"));
		part[i].rot = {};
	    part[i].rot[1] = Utils.getNoNil(x, 0);
	    part[i].rot[2] = Utils.getNoNil(y, 0);
	    part[i].rot[3] = Utils.getNoNil(z, 0);
		for j=1, 3 do
			if part[i].rot[j] ~= 0 then
				part[i].rot[j] = 1/part[i].rot[j]
			end;
		end;
    end;
	
	self.dustParticleSystems = {};
    local dustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.dustParticleSystems#count"), 0);
    for i=1, dustParticleSystemCount do
        local namei = string.format("vehicle.dustParticleSystems.part%d", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
        Utils.loadParticleSystem(xmlFile, self.dustParticleSystems, namei, nodei, false, nil, self.baseDirectory)			
    end;
			
	
	self.keyType = InputBinding.getButtonKeyName;
	self.keyName = "Taste";	
	if g_settingsJoystickEnabled then 
		self.keySet = not self.keySet;
	end; 	
	
	
	self.moveParticle = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.moveParticle#index"));
	
	self.vehicleType = getXMLString(xmlFile, "vehicle#type");
	self.AIon = false; 
	
	self.aiLeftMarker12 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.aiExtend#LL"));
    self.aiRightMarker12 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.aiExtend#RR"));
	self.saveAILM12 = self.aiLeftMarker12;
	self.saveAIRM12 = self.aiRightMarker12;
	
	self.aiLeftMarker6 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.aiExtend#L"));
    self.aiRightMarker6 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.aiExtend#R"));
	self.saveAILM6 = self.aiLeftMarker6;
	self.saveAIRM6 = self.aiRightMarker6;
	
	self.saveAILM = self.aiLeftMarker;
	self.saveAIRM = self.aiRightMarker;
	self.aiRightMarker = nil; 
	self.aiLeftMarker = nil; 
	self.change = 0; 
	self.dy = 6000; 
	
end;

function AmazoneZAOC900:keyEvent(unicode, sym, modifier, isDown)
	if self:getIsActiveForInput() and InputBinding.isPressed(InputBinding.AmazoneZAOC900_ON) and self.AIon == false and self.fillLevel > 0 then
		self.turnOn = not self.turnOn; 
	end;
	
	if self:getIsActiveForInput() and InputBinding.isPressed(InputBinding.AmazoneZAOC900_CHANGE) and self.AIon == false and self.fillLevel > 0 then
		self.change = self.change + 1;  
		if self.change == 3 then
			self.change = 0; 
		end; 	
	end;
	
	if isDown and sym==Input.KEY_8 then
		self.keySet = not self.keySet; 
	end; 	
	
end;


function AmazoneZAOC900:update(dt)

	if self.attacherVehicle then
	
		if self.keySet then
			self.keyType = InputBinding.getButtonName;
			self.keyName = "Button"; 
		else
			self.keyType = InputBinding.getButtonKeyName;
			self.keyName = "Taste"; 
		end; 
		
		if self.fixAttacher then
			local implement = self.attacherVehicle:getImplementByObject(self);
			if implement ~= nil then
				local joint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.saveJoint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.maxRotJoint = joint.maxRotLimit;
				self.attacherJointIndex = implement.jointDescIndex;
				self.rotationCopy  = joint.minRot;
				joint.minRot = {Utils.degToRad(2),0,0};			
				joint.maxRotLimit = {0, 0, 0};
				setJointRotationLimit(joint.jointIndex, 2, true, 0, 0);
			end;
			self.fixAttacher = false;
		end;
	
		if self.attacherVehicle.isAITractorActivated ~= nil then
			self.AIon = self.attacherVehicle.isAITractorActivated;
		end; 
		
		if self.AIon == false then
			self.AIblock = nil;
		end; 	
			
		if self.delay == true then
	        self.dy = self.dy - dt;
	        if self.dy <= 0 then
	            self.delay = false;
	            self.dy = 6000; 
				self.AIblock = nil; 
	        end;
	    end;
		
		if self.turnOn then
			if self.attacherVehicle.isAITractorActivated ~= nil then
				if self.change == 0 then
					self.aiLeftMarker = self.saveAILM; 
					self.aiRightMarker = self.saveAIRM; 
				end; 
				if self.change == 1 then
					self.aiLeftMarker = self.saveAILM12; 
					self.aiRightMarker = self.saveAIRM12; 
				end; 
				if self.change == 2 then
					self.aiLeftMarker = self.saveAILM6; 
					self.aiRightMarker = self.saveAIRM6; 
				end; 
				AITractor.updateToolsInfo(self.attacherVehicle)
			end; 	
			for i=1, table.getn(self.drums) do
				rotate(self.drums[i].index, self.drums[i].rot[1] / 10 * dt, self.drums[i].rot[2] / 10 * dt, self.drums[i].rot[3] / 10 * dt);
			end;
			
			if self.fillLevel == 0 then
				self.turnOn = not self.turnOn;	
			end; 
			
			if self.AIon and self.AIblock == true then
				Utils.setEmittingState(self.dustParticleSystems, false)
			end; 
			if self.fillLevel > 0 and self.AIblock == nil then
			

				Utils.setEmittingState(self.dustParticleSystems, true)
				
				if self.change == 0 then
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					local deltaLevel = FruitUtil.fruitIndexToDesc[self.currentFillType].literPerQm * difficultyMultiplier * 2; 
					self:setFillLevel(self.fillLevel-deltaLevel, self.currentFillType);
					local x,y,z = getWorldTranslation(self.cuttingAreas[1].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[1].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[1].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
					local x,y,z = getWorldTranslation(self.cuttingAreas[2].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[2].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[2].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
	           end; 
			   
			   	if self.change == 1 then
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					local deltaLevel = FruitUtil.fruitIndexToDesc[self.currentFillType].literPerQm * difficultyMultiplier * 1.5; 
					self:setFillLevel(self.fillLevel-deltaLevel, self.currentFillType);
			        local x,y,z = getWorldTranslation(self.cuttingAreas[3].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[3].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[3].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
					local x,y,z = getWorldTranslation(self.cuttingAreas[4].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[4].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[4].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
	           end; 
			   
			   
			   	if self.change == 2 then
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					local deltaLevel = FruitUtil.fruitIndexToDesc[self.currentFillType].literPerQm * difficultyMultiplier * 1; 
					self:setFillLevel(self.fillLevel-deltaLevel, self.currentFillType);
				    local x,y,z = getWorldTranslation(self.cuttingAreas[5].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[5].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[5].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
					local x,y,z = getWorldTranslation(self.cuttingAreas[6].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[6].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[6].height);
	                Utils.updateSprayArea(x, z, x1, z1, x2, z2);
	           end; 
				
			end;
		else
			Utils.setEmittingState(self.dustParticleSystems, false)
			if self.attacherVehicle.isAITractorActivated ~= nil then
				self.aiLeftMarker = nil; 
				self.aiRightMarker = nil;
				AITractor.updateToolsInfo(self.attacherVehicle)
			end; 	
		end;
		
	end; 
	
end;

function AmazoneZAOC900:draw()

	if self.AIon == false then
		if self.fillLevel > 0 then
			if self.turnOn then
				g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.AmazoneZAOC900_ON).. ": " ..g_i18n:getText("AmazoneZAOC900_2")); 
			else
				g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.AmazoneZAOC900_ON).. ": " ..g_i18n:getText("AmazoneZAOC900_1"));
			end; 	
			
			if self.change == 0 then
				g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.AmazoneZAOC900_CHANGE).. ": " ..g_i18n:getText("AmazoneZAOC900_3").. " 18m");
			end; 
			if self.change == 1 then
				g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.AmazoneZAOC900_CHANGE).. ": " ..g_i18n:getText("AmazoneZAOC900_3").. " 12m");
			end
			if self.change == 2 then
				g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.AmazoneZAOC900_CHANGE).. ": " ..g_i18n:getText("AmazoneZAOC900_3").. " 6m");
			end
		end; 
		
		if self.fillLevel == 0 then
			g_currentMission:addExtraPrintText(g_i18n:getText("AmazoneZAOC900_4"));
		end; 

	end;

end;


function AmazoneZAOC900:onAttach()
	self.fixAttacher = true; 
end; 	


function AmazoneZAOC900:onDetach()
	local joint = self.saveJoint; 
	joint.minRot = self.rotationCopy;
	joint.maxRotLimit = self.maxRotJoint;
	Utils.setEmittingState(self.dustParticleSystems, false)
end;


function AmazoneZAOC900:delete()
	Utils.setEmittingState(self.dustParticleSystems, false)
	Utils.deleteParticleSystem(self.dustParticleSystems)
end;


function AmazoneZAOC900:mouseEvent(posX, posY, isDown, isUp, button)
end;


function AmazoneZAOC900:aiLower()
	if self.change == 2 then
		self.dy = 2000; 
	end; 
	if self.change == 1 then
		self.dy = 1000; 
	end; 
	self.delay = true;
end; 


function AmazoneZAOC900:aiRaise()
	self.AIblock = true; 
end; 

